--Setup pagesize parameters
SET NEWPAGE 0
SET PAGESIZE 55

--Set the linesize, which must match the number of equal signs used
--for the ruling lines in the headers and footers.
SET LINESIZE 75

TTITLE CENTER 'The Fictional Company' SKIP 2 -
       CENTER 'Employee Turnover Report' SKIP 1-
       LEFT '===================================' -
            '=======================================' -
       SKIP 3

--Format the columns
CLEAR COLUMNS
COLUMN sort_column NOPRINT
COLUMN employee_id NOPRINT
COLUMN employee_status_text HEADING 'Status' FORMAT A29
COLUMN employee_name HEADING 'Employee Name' FORMAT A20
COLUMN employee_hire_date HEADING 'Hire Date' FORMAT A11
COLUMN employee_termination_date HEADING 'Term Date' FORMAT A11

--Breaks and computations
BREAK ON employee_status_text SKIP 2 NODUPLICATES
CLEAR COMPUTES
COMPUTE NUMBER LABEL 'Total Count' OF employee_name ON employee_status_text

--Set the date format to use
ALTER SESSION SET NLS_DATE_FORMAT = 'dd-Mon-yyyy';

SELECT 1 sort_column,
       'Employed at Beginning of Year' employee_status_text,
       employee_id,
       employee_name,
       employee_hire_date,
       employee_termination_date
  FROM employee
 WHERE employee_hire_date < TO_DATE('1-Jan-1998','dd-mon-yyyy')
   AND (employee_termination_date IS NULL
        OR employee_termination_date >= TO_DATE('1-Jan-1998','dd-mon-yyyy'))
UNION ALL
SELECT 2 as sort_column,
       'Hired During Year' as employee_status_text,
       employee_id,
       employee_name,
       employee_hire_date,
       employee_termination_date
  FROM employee
 WHERE employee_hire_date >= TO_DATE('1-Jan-1998','dd-mon-yyyy')
   AND (employee_hire_date < TO_DATE('1-Jan-1999','dd-mon-yyyy'))
UNION ALL
SELECT 3 as sort_column,
       'Terminated During Year' as employee_status_text,
       employee_id,
       employee_name,
       employee_hire_date,
       employee_termination_date
  FROM employee
 WHERE employee_termination_date >= TO_DATE('1-Jan-1998','dd-mon-yyyy')
   AND (employee_termination_date < TO_DATE('1-Jan-1999','dd-mon-yyyy'))
UNION ALL
SELECT 4 as sort_column,
       'Employed at End of Year' as employee_status_text,
       employee_id,
       employee_name,
       employee_hire_date,
       employee_termination_date
  FROM employee
 WHERE employee_hire_date < TO_DATE('1-Jan-1999','dd-mon-yyyy')
   AND (employee_termination_date IS NULL
        OR employee_termination_date >= TO_DATE('1-Jan-1999','dd-mon-yyyy'))
ORDER BY sort_column, employee_id, employee_hire_date;


